SIGNATURE1      equ  2387h
SIGNATURE2      equ  7A8Dh
code            segment
                assume cs:code
                org 100h               ;Start of .COM program.
start:          jmp short check_install ;Jump to TSR installation routine.
old_int9        label dword
offset_int9     dw 0
segment_int9    dw 0
State           db 1                   ;1: Enabled; 0: Not enabled.
SIGNATURE       dw SIGNATURE1,SIGNATURE2
;
;  Interrupt 9 handler (keyboard)
;
int9handler:    cmp State,0            ;Is sound enabled?
                jz go_to_old_int9      ;Jump if it is not.
                push ax                ;Preserve used registers.
		push cx
		push dx
                mov al,182             ;Generate square wave for timer 2
                out 67,al              ;in order to produce sound.
                in al,96               ;Read keyboard controller.
                cmp al,128             ;Check if a key was pressed.
                jnc silence            ;Jump if it was released.
                add al,al              ;Adjust scan code: for each two values
                jns find_freq          ;a semitone must be incremented.
                sub al,127             ;
find_freq:      mov cl,al              ;CX = Number of half semitones above
                xor ch,ch              ;     LA (55 Hz).
                mov ax,21690           ;AX = 55 Hz (LA)
loop_freq:      mov dx,63670           ;DX = Half semitone.
                mul dx                 ;DX = Next half semitone.
                mov ax,dx              ;AX = Next half semitone.
                loop loop_freq         ;Continue calculating frequency.
                jmp short sound        ;Set note.
silence:        mov ax,40              ;Silence.
sound:          out 66,al              ;Set low byte of counter.
		mov al,ah
                out 66,al              ;Set high byte of counter.
                in al,97               ;Enable speaker output.
		or al,3
		out 97,al
                pop dx                 ;Restore used registers.
		pop cx
		pop ax
go_to_old_int9:
                jmp old_int9           ;Go to old keyboard interrupt handler.
;
; TSR install.
;
check_install:  xor ax,ax              ;Point to interrupt vector table.
                mov es,ax
                lds bx,es:9*4          ;DS:BX = Pointer to old keyboard
                                       ;interrupt handler.
                cmp word ptr [bx-4],SIGNATURE1  ;If SIGNATURE does not match,
                jne install                     ;install TSR.
                cmp word ptr [bx-2],SIGNATURE2  ;If SIGNATURE does not match,
                jne install                     ;install TSR.
                xor ds:State,1         ;Change State: sound yes/no.
		push cs
                pop ds                 ;DS = Code segment.
                mov dx,offset sound_yes ;Point to sound active text.
                jnz show_text
                mov dx,offset sound_no ;Point to sound inactive text.
show_text:      mov ah,9
                int 21h                ;Call DOS function to show it.
                in al,97               ;Disable sound.
		and al,252
		out 97,al
                mov ax,4c00h           ;End program.
		int 21h
install:        mov offset_int9,bx     ;Preserve old interrupt 9 handler.
                mov segment_int9,ds    ;Preserve segment.
                cli                    ;Disable interrupt.
                mov es:9*4,offset int9handler ;New interrupt 9 offset.
                mov es:9*4+2,cs        ;New interrupt 9 segment.
                sti                    ;Enable interrupts.
                push cs
                pop ds                  ;DS = Code segment.
                mov dx,offset sound_yes ;Point to sound active text.
		mov ah,9
		int 21h
                mov dx,offset check_install  ;Install TSR.
		int 27h
sound_yes       db "Sound active.",13,10,"$"
sound_no        db "Sound inactive.",13,10,"$"
code            ends
                end start
